import os
import warnings
from typing import Literal, Optional

from pydantic import BaseModel, ConfigDict, model_validator


class Model(BaseModel):
    """
    Model information.

    Attributes:
        id: Unique identifier for the model, passed as model parameter for requests
        model_type: API type
        client: Client name
        endpoint: Custom endpoint for the model
        aliases: List of aliases for the model
        supports_tools: Whether the model supports tool calling
        supports_structured_output: Whether the model supports structured output
        supports_thinking: Whether the model supports configuring thinking
            on/off through system message or request parameters
        thinking_prefix: System message prefix when thinking is enabled
            (tag-based)
        no_thinking_prefix: System message prefix when thinking is disabled
            (tag-based)
        thinking_param_enable: Dict of parameters to apply when thinking is
            enabled (param-based)
        thinking_param_disable: Dict of parameters to apply when thinking is
            disabled (param-based)

    Thinking mode can be enabled via two mechanisms:
        1. Tag-based: Use thinking_prefix/no_thinking_prefix (appended to
            system message)
        2. Param-based: Use thinking_param_enable/thinking_param_disable
            (merged into request params)

    All aliases are deprecated and will trigger a warning when used.
    """

    # https://docs.pydantic.dev/latest/api/config/#protected-namespaces
    # we disable this protection to avoid warnings from `model_type`
    model_config = ConfigDict(
        protected_namespaces=(),
    )

    id: str

    # why do we have a model_type? because ChatNVIDIA can speak both chat and vlm.
    model_type: Optional[
        Literal["chat", "vlm", "nv-vlm", "embedding", "ranking", "completions", "qa"]
    ] = None

    client: Optional[
        Literal["ChatNVIDIA", "NVIDIAEmbeddings", "NVIDIARerank", "NVIDIA"]
    ] = None

    endpoint: Optional[str] = None

    aliases: Optional[list] = None

    supports_tools: Optional[bool] = False

    supports_structured_output: Optional[bool] = False

    supports_thinking: Optional[bool] = False

    thinking_prefix: Optional[str] = None

    no_thinking_prefix: Optional[str] = None

    thinking_param_enable: Optional[dict] = None

    thinking_param_disable: Optional[dict] = None

    base_model: Optional[str] = None

    def __hash__(self) -> int:
        return hash(self.id)

    @model_validator(mode="after")
    def validate_client(self) -> "Model":
        if self.client:
            supported = {
                "ChatNVIDIA": ("chat", "vlm", "nv-vlm", "qa"),
                "NVIDIAEmbeddings": ("embedding",),
                "NVIDIARerank": ("ranking",),
                "NVIDIA": ("completions",),
            }
            if self.model_type not in supported.get(self.client, ()):
                raise ValueError(
                    f"Model type '{self.model_type}' not supported "
                    f"by client '{self.client}'"
                )
        return self

    @model_validator(mode="after")
    def validate_thinking_config(self) -> "Model":
        """Warn if both param-based and tag-based thinking are configured."""
        has_param_based = (
            self.thinking_param_enable is not None
            or self.thinking_param_disable is not None
        )
        has_tag_based = (
            self.thinking_prefix is not None or self.no_thinking_prefix is not None
        )

        if has_param_based and has_tag_based:
            warnings.warn(
                f"Model '{self.id}' has both param-based thinking "
                f"(thinking_param_enable/disable) and tag-based thinking "
                f"(thinking_prefix/no_thinking_prefix) configured. "
                f"Param-based thinking will take precedence and tag-based "
                f"thinking will be ignored.",
                UserWarning,
                stacklevel=2,
            )
        return self


CHAT_MODEL_TABLE = {
    "google/gemma-7b": Model(
        id="google/gemma-7b",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-gemma-7b", "playground_gemma_7b", "gemma_7b"],
    ),
    "mistralai/mistral-7b-instruct-v0.2": Model(
        id="mistralai/mistral-7b-instruct-v0.2",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-mistral-7b-instruct-v2", "playground_mistral_7b", "mistral_7b"],
    ),
    "mistralai/mixtral-8x7b-instruct-v0.1": Model(
        id="mistralai/mixtral-8x7b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-mixtral-8x7b-instruct", "playground_mixtral_8x7b", "mixtral_8x7b"],
    ),
    "google/codegemma-7b": Model(
        id="google/codegemma-7b",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-codegemma-7b"],
    ),
    "gotocompany/gemma-2-9b-cpt-sahabatai-instruct": Model(
        id="gotocompany/gemma-2-9b-cpt-sahabatai-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mistralai/mixtral-8x22b-instruct-v0.1": Model(
        id="mistralai/mixtral-8x22b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-mixtral-8x22b-instruct"],
    ),
    "mistralai/mixtral-8x22b-v0.1": Model(
        id="mistralai/mixtral-8x22b-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "meta/llama3-8b-instruct": Model(
        id="meta/llama3-8b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-llama3-8b"],
    ),
    "meta/llama3-70b-instruct": Model(
        id="meta/llama3-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-llama3-70b"],
    ),
    "microsoft/phi-3-mini-128k-instruct": Model(
        id="microsoft/phi-3-mini-128k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-mini"],
    ),
    "microsoft/phi-3-mini-4k-instruct": Model(
        id="microsoft/phi-3-mini-4k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-mini-4k", "playground_phi2", "phi2"],
    ),
    "aisingapore/sea-lion-7b-instruct": Model(
        id="aisingapore/sea-lion-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-sea-lion-7b-instruct"],
    ),
    "microsoft/phi-3-small-8k-instruct": Model(
        id="microsoft/phi-3-small-8k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-small-8k-instruct"],
    ),
    "microsoft/phi-3-small-128k-instruct": Model(
        id="microsoft/phi-3-small-128k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-small-128k-instruct"],
    ),
    "microsoft/phi-3-medium-4k-instruct": Model(
        id="microsoft/phi-3-medium-4k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-medium-4k-instruct"],
    ),
    "ibm/granite-3.3-8b-instruct": Model(
        id="ibm/granite-3.3-8b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        supports_tools=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
    "ibm/granite-guardian-3.0-8b": Model(
        id="ibm/granite-guardian-3.0-8b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mediatek/breeze-7b-instruct": Model(
        id="mediatek/breeze-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-breeze-7b-instruct"],
    ),
    "upstage/solar-10.7b-instruct": Model(
        id="upstage/solar-10.7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-solar-10_7b-instruct"],
    ),
    "mistralai/mistral-7b-instruct-v0.3": Model(
        id="mistralai/mistral-7b-instruct-v0.3",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-mistral-7b-instruct-v03"],
    ),
    "01-ai/yi-large": Model(
        id="01-ai/yi-large",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-yi-large"],
    ),
    "mistralai/codestral-22b-instruct-v0.1": Model(
        id="mistralai/codestral-22b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-codestral-22b-instruct-v01"],
        supports_structured_output=True,
    ),
    "google/gemma-2-9b-it": Model(
        id="google/gemma-2-9b-it",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-gemma-2-9b-it"],
    ),
    "google/gemma-2-27b-it": Model(
        id="google/gemma-2-27b-it",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-gemma-2-27b-it"],
    ),
    "google/gemma-3-12b-it": Model(
        id="google/gemma-3-12b-it",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "google/gemma-3-1b-it": Model(
        id="google/gemma-3-1b-it",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "google/gemma-3-27b-it": Model(
        id="google/gemma-3-27b-it",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "google/gemma-3-4b-it": Model(
        id="google/gemma-3-4b-it",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "google/gemma-3n-e2b-it": Model(
        id="google/gemma-3n-e2b-it",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "google/gemma-3n-e4b-it": Model(
        id="google/gemma-3n-e4b-it",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "google/shieldgemma-9b": Model(
        id="google/shieldgemma-9b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "microsoft/phi-3-medium-128k-instruct": Model(
        id="microsoft/phi-3-medium-128k-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        aliases=["ai-phi-3-medium-128k-instruct"],
    ),
    "nv-mistralai/mistral-nemo-12b-instruct": Model(
        id="nv-mistralai/mistral-nemo-12b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "meta/llama-3.1-8b-instruct": Model(
        id="meta/llama-3.1-8b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "meta/llama-3.1-70b-instruct": Model(
        id="meta/llama-3.1-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "meta/llama-3.1-405b-instruct": Model(
        id="meta/llama-3.1-405b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "meta/llama-guard-4-12b": Model(
        id="meta/llama-guard-4-12b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mistralai/mamba-codestral-7b-v0.1": Model(
        id="mistralai/mamba-codestral-7b-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "google/gemma-2-2b-it": Model(
        id="google/gemma-2-2b-it",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mistralai/magistral-small-2506": Model(
        id="mistralai/magistral-small-2506",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mistralai/mistral-medium-3-instruct": Model(
        id="mistralai/mistral-medium-3-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "mistralai/mistral-nemotron": Model(
        id="mistralai/mistral-nemotron",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
    ),
    "mistralai/mistral-small-24b-instruct": Model(
        id="mistralai/mistral-small-24b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "mistralai/mistral-small-3.1-24b-instruct-2503": Model(
        id="mistralai/mistral-small-3.1-24b-instruct-2503",
        model_type="vlm",
        client="ChatNVIDIA",
        supports_tools=True,
    ),
    "mistralai/mathstral-7b-v0.1": Model(
        id="mistralai/mathstral-7b-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "rakuten/rakutenai-7b-instruct": Model(
        id="rakuten/rakutenai-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "rakuten/rakutenai-7b-chat": Model(
        id="rakuten/rakutenai-7b-chat",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "baichuan-inc/baichuan2-13b-chat": Model(
        id="baichuan-inc/baichuan2-13b-chat",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "thudm/chatglm3-6b": Model(
        id="thudm/chatglm3-6b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "tiiuae/falcon3-7b-instruct": Model(
        id="tiiuae/falcon3-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "microsoft/phi-3.5-mini-instruct": Model(
        id="microsoft/phi-3.5-mini-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "microsoft/phi-4-mini-flash-reasoning": Model(
        id="microsoft/phi-4-mini-flash-reasoning",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "microsoft/phi-4-mini-instruct": Model(
        id="microsoft/phi-4-mini-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
    ),
    "microsoft/phi-4-multimodal-instruct": Model(
        id="microsoft/phi-4-multimodal-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "utter-project/eurollm-9b-instruct": Model(
        id="utter-project/eurollm-9b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/nemotron-mini-4b-instruct": Model(
        id="nvidia/nemotron-mini-4b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "yentinglin/llama-3-taiwan-70b-instruct": Model(
        id="yentinglin/llama-3-taiwan-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "tokyotech-llm/llama-3-swallow-70b-instruct-v0.1": Model(
        id="tokyotech-llm/llama-3-swallow-70b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "abacusai/dracarys-llama-3.1-70b-instruct": Model(
        id="abacusai/dracarys-llama-3.1-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "qwen/qwen2-7b-instruct": Model(
        id="qwen/qwen2-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "meta/llama-3.2-1b-instruct": Model(
        id="meta/llama-3.2-1b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "meta/llama-3.2-3b-instruct": Model(
        id="meta/llama-3.2-3b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "institute-of-science-tokyo/llama-3.1-swallow-8b-instruct-v0.1": Model(
        id="institute-of-science-tokyo/llama-3.1-swallow-8b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_structured_output=True,
    ),
    "institute-of-science-tokyo/llama-3.1-swallow-70b-instruct-v0.1": Model(
        id="institute-of-science-tokyo/llama-3.1-swallow-70b-instruct-v0.1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_structured_output=True,
    ),
    "igenius/colosseum_355b_instruct_16k": Model(
        id="igenius/colosseum_355b_instruct_16k",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "igenius/italia_10b_instruct_16k": Model(
        id="igenius/italia_10b_instruct_16k",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "zyphra/zamba2-7b-instruct": Model(
        id="zyphra/zamba2-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/nemotron-4-mini-hindi-4b-instruct": Model(
        id="nvidia/nemotron-4-mini-hindi-4b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_structured_output=True,
    ),
    "nvidia/llama-3.1-nemoguard-8b-content-safety": Model(
        id="nvidia/llama-3.1-nemoguard-8b-content-safety",
        model_type="chat",
        client="ChatNVIDIA",
        supports_structured_output=True,
    ),
    "nvidia/llama-3.1-nemoguard-8b-topic-control": Model(
        id="nvidia/llama-3.1-nemoguard-8b-topic-control",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/usdcode-llama-3.1-70b-instruct": Model(
        id="nvidia/usdcode-llama-3.1-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "meta/llama-3.3-70b-instruct": Model(
        id="meta/llama-3.3-70b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "qwen/qwen2.5-coder-32b-instruct": Model(
        id="qwen/qwen2.5-coder-32b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "qwen/qwen2.5-coder-7b-instruct": Model(
        id="qwen/qwen2.5-coder-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "qwen/qwen2.5-7b-instruct": Model(
        id="qwen/qwen2.5-7b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "qwen/qwen3-235b-a22b": Model(
        id="qwen/qwen3-235b-a22b",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        supports_tools=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
    "qwen/qwq-32b": Model(
        id="qwen/qwq-32b",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
    ),
    "nvidia/llama-3.1-nemotron-70b-reward": Model(
        id="nvidia/llama-3.1-nemotron-70b-reward",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "deepseek-ai/deepseek-r1": Model(
        id="deepseek-ai/deepseek-r1",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "deepseek-ai/deepseek-r1-0528": Model(
        id="deepseek-ai/deepseek-r1-0528",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
    ),
    "deepseek-ai/deepseek-r1-distill-llama-8b": Model(
        id="deepseek-ai/deepseek-r1-distill-llama-8b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "deepseek-ai/deepseek-r1-distill-qwen-14b": Model(
        id="deepseek-ai/deepseek-r1-distill-qwen-14b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "deepseek-ai/deepseek-r1-distill-qwen-32b": Model(
        id="deepseek-ai/deepseek-r1-distill-qwen-32b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "deepseek-ai/deepseek-r1-distill-qwen-7b": Model(
        id="deepseek-ai/deepseek-r1-distill-qwen-7b",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/llama-3.1-nemotron-nano-8b-v1": Model(
        id="nvidia/llama-3.1-nemotron-nano-8b-v1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_prefix="detailed thinking on",
        no_thinking_prefix="detailed thinking off",
    ),
    "nvidia/llama-3.1-nemotron-nano-4b-v1.1": Model(
        id="nvidia/llama-3.1-nemotron-nano-4b-v1.1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_prefix="detailed thinking on",
        no_thinking_prefix="detailed thinking off",
        supports_tools=True,
    ),
    "nvidia/llama-3.1-nemotron-ultra-253b-v1": Model(
        id="nvidia/llama-3.1-nemotron-ultra-253b-v1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_prefix="detailed thinking on",
        no_thinking_prefix="detailed thinking off",
        supports_tools=True,
    ),
    "nvidia/llama-3.3-nemotron-super-49b-v1": Model(
        id="nvidia/llama-3.3-nemotron-super-49b-v1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_prefix="detailed thinking on",
        no_thinking_prefix="detailed thinking off",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "nvidia/llama-3.3-nemotron-super-49b-v1.5": Model(
        id="nvidia/llama-3.3-nemotron-super-49b-v1.5",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_prefix="/think",
        no_thinking_prefix="/no_think",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "moonshotai/kimi-k2-instruct": Model(
        id="moonshotai/kimi-k2-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "openai/gpt-oss-20b": Model(
        id="openai/gpt-oss-20b",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "openai/gpt-oss-120b": Model(
        id="openai/gpt-oss-120b",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "marin/marin-8b-instruct": Model(
        id="marin/marin-8b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "speakleash/bielik-11b-v2.3-instruct": Model(
        id="speakleash/bielik-11b-v2.3-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/nvidia-nemotron-nano-9b-v2": Model(
        id="nvidia/nvidia-nemotron-nano-9b-v2",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
        supports_thinking=True,
        thinking_prefix="/think",
        no_thinking_prefix="/no_think",
    ),
    "deepseek-ai/deepseek-v3.1": Model(
        id="deepseek-ai/deepseek-v3.1",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
    "bytedance/seed-oss-36b-instruct": Model(
        id="bytedance/seed-oss-36b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "moonshotai/kimi-k2-instruct-0905": Model(
        id="moonshotai/kimi-k2-instruct-0905",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "qwen/qwen3-next-80b-a3b-instruct": Model(
        id="qwen/qwen3-next-80b-a3b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "qwen/qwen3-next-80b-a3b-thinking": Model(
        id="qwen/qwen3-next-80b-a3b-thinking",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "deepseek-ai/deepseek-v3.1-terminus": Model(
        id="deepseek-ai/deepseek-v3.1-terminus",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
        supports_thinking=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
    "minimaxai/minimax-m2": Model(
        id="minimaxai/minimax-m2",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "moonshotai/kimi-k2-thinking": Model(
        id="moonshotai/kimi-k2-thinking",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
    ),
    "nvidia/nemotron-3-nano-30b-a3b": Model(
        id="nvidia/nemotron-3-nano-30b-a3b",
        model_type="chat",
        client="ChatNVIDIA",
        supports_tools=True,
        supports_structured_output=True,
        supports_thinking=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
    "deepseek-ai/deepseek-v3.2": Model(
        id="deepseek-ai/deepseek-v3.2",
        model_type="chat",
        client="ChatNVIDIA",
        supports_thinking=True,
        supports_structured_output=True,
        supports_tools=True,
        thinking_param_enable={"chat_template_kwargs": {"enable_thinking": True}},
        thinking_param_disable={"chat_template_kwargs": {"enable_thinking": False}},
    ),
}

QA_MODEL_TABLE = {
    "nvidia/llama3-chatqa-1.5-8b": Model(
        id="nvidia/llama3-chatqa-1.5-8b",
        model_type="qa",
        client="ChatNVIDIA",
        aliases=["ai-chatqa-1.5-8b"],
    ),
}

VLM_MODEL_TABLE = {
    "google/paligemma": Model(
        id="google/paligemma",
        model_type="nv-vlm",
        client="ChatNVIDIA",
        endpoint="https://ai.api.nvidia.com/v1/vlm/google/paligemma",
        aliases=["ai-google-paligemma"],
    ),
    "microsoft/phi-3-vision-128k-instruct": Model(
        id="microsoft/phi-3-vision-128k-instruct",
        model_type="nv-vlm",
        client="ChatNVIDIA",
        endpoint="https://ai.api.nvidia.com/v1/vlm/microsoft/phi-3-vision-128k-instruct",
        aliases=["ai-phi-3-vision-128k-instruct"],
    ),
    "microsoft/phi-3.5-vision-instruct": Model(
        id="microsoft/phi-3.5-vision-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "nvidia/nvclip": Model(
        id="nvidia/nvclip",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "nvidia/nemoretriever-parse": Model(
        id="nvidia/nemoretriever-parse",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "nvidia/riva-translate-4b-instruct": Model(
        id="nvidia/riva-translate-4b-instruct",
        model_type="chat",
        client="ChatNVIDIA",
    ),
    "meta/llama-3.2-11b-vision-instruct": Model(
        id="meta/llama-3.2-11b-vision-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
        endpoint="https://ai.api.nvidia.com/v1/gr/meta/llama-3.2-11b-vision-instruct/chat/completions",
    ),
    "meta/llama-3.2-90b-vision-instruct": Model(
        id="meta/llama-3.2-90b-vision-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
        endpoint="https://ai.api.nvidia.com/v1/gr/meta/llama-3.2-90b-vision-instruct/chat/completions",
    ),
    "meta/llama-4-maverick-17b-128e-instruct": Model(
        id="meta/llama-4-maverick-17b-128e-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "meta/llama-4-scout-17b-16e-instruct": Model(
        id="meta/llama-4-scout-17b-16e-instruct",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "nvidia/llama-3.1-nemotron-nano-vl-8b-v1": Model(
        id="nvidia/llama-3.1-nemotron-nano-vl-8b-v1",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "mistralai/ministral-14b-instruct-2512": Model(
        id="mistralai/ministral-14b-instruct-2512",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
    "mistralai/mistral-large-3-675b-instruct-2512": Model(
        id="mistralai/mistral-large-3-675b-instruct-2512",
        model_type="vlm",
        client="ChatNVIDIA",
    ),
}

EMBEDDING_MODEL_TABLE = {
    "snowflake/arctic-embed-l": Model(
        id="snowflake/arctic-embed-l",
        model_type="embedding",
        client="NVIDIAEmbeddings",
        aliases=["ai-arctic-embed-l"],
    ),
    "NV-Embed-QA": Model(
        id="NV-Embed-QA",
        model_type="embedding",
        client="NVIDIAEmbeddings",
        endpoint="https://ai.api.nvidia.com/v1/retrieval/nvidia/embeddings",
        aliases=[
            "ai-embed-qa-4",
        ],
    ),
    "nvidia/nv-embed-v1": Model(
        id="nvidia/nv-embed-v1",
        model_type="embedding",
        client="NVIDIAEmbeddings",
        aliases=["ai-nv-embed-v1"],
    ),
    "nvidia/nv-embedqa-mistral-7b-v2": Model(
        id="nvidia/nv-embedqa-mistral-7b-v2",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/nv-embedqa-e5-v5": Model(
        id="nvidia/nv-embedqa-e5-v5",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "baai/bge-m3": Model(
        id="baai/bge-m3",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/llama-3.2-nv-embedqa-1b-v1": Model(
        id="nvidia/llama-3.2-nv-embedqa-1b-v1",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/llama-3.2-nv-embedqa-1b-v2": Model(
        id="nvidia/llama-3.2-nv-embedqa-1b-v2",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/llama-3.2-nemoretriever-1b-vlm-embed-v1": Model(
        id="nvidia/llama-3.2-nemoretriever-1b-vlm-embed-v1",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/nv-embedcode-7b-v1": Model(
        id="nvidia/nv-embedcode-7b-v1",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
    "nvidia/llama-3.2-nemoretriever-300m-embed-v2": Model(
        id="nvidia/llama-3.2-nemoretriever-300m-embed-v2",
        model_type="embedding",
        client="NVIDIAEmbeddings",
    ),
}

RANKING_MODEL_TABLE = {
    "nv-rerank-qa-mistral-4b:1": Model(
        id="nv-rerank-qa-mistral-4b:1",
        model_type="ranking",
        client="NVIDIARerank",
        endpoint="https://ai.api.nvidia.com/v1/retrieval/nvidia/reranking",
        aliases=["ai-rerank-qa-mistral-4b"],
    ),
    "nvidia/llama-3.2-nv-rerankqa-1b-v1": Model(
        id="nvidia/llama-3.2-nv-rerankqa-1b-v1",
        model_type="ranking",
        client="NVIDIARerank",
        endpoint="https://ai.api.nvidia.com/v1/retrieval/nvidia/llama-3_2-nv-rerankqa-1b-v1/reranking",
    ),
    "nvidia/llama-3.2-nv-rerankqa-1b-v2": Model(
        id="nvidia/llama-3.2-nv-rerankqa-1b-v2",
        model_type="ranking",
        client="NVIDIARerank",
        endpoint="https://ai.api.nvidia.com/v1/retrieval/nvidia/llama-3_2-nv-rerankqa-1b-v2/reranking",
    ),
}

COMPLETION_MODEL_TABLE = {
    "bigcode/starcoder2-7b": Model(
        id="bigcode/starcoder2-7b",
        model_type="completions",
        client="NVIDIA",
    ),
    "nvidia/mistral-nemo-minitron-8b-base": Model(
        id="nvidia/mistral-nemo-minitron-8b-base",
        model_type="completions",
        client="NVIDIA",
    ),
}


OPENAI_MODEL_TABLE = {
    "gpt-3.5-turbo": Model(
        id="gpt-3.5-turbo",
        model_type="chat",
        client="ChatNVIDIA",
        endpoint="https://api.openai.com/v1/chat/completions",
        supports_tools=True,
    ),
}


MODEL_TABLE = {
    **CHAT_MODEL_TABLE,
    **QA_MODEL_TABLE,
    **VLM_MODEL_TABLE,
    **EMBEDDING_MODEL_TABLE,
    **RANKING_MODEL_TABLE,
    **COMPLETION_MODEL_TABLE,
}

if "_INCLUDE_OPENAI" in os.environ:
    MODEL_TABLE.update(OPENAI_MODEL_TABLE)


def register_model(model: Model) -> None:
    """Register a model as a known model.

    Must be done at the beginning of a program, at least before the model is used or
    available models are listed.

    For instance:

    ```python
    from langchain_nvidia_ai_endpoints import register_model, Model


    register_model(
        Model(
            id="my-custom-model-name",
            model_type="chat",
            client="ChatNVIDIA",
            endpoint="http://host:port/path-to-my-model"
        )
    )
    llm = ChatNVIDIA(model="my-custom-model-name")
    ```

    Be sure that the `id` matches the model parameter the endpoint expects.

    Supported model types are chat models, which must accept and produce chat completion
    payloads.

    Supported model clients are `ChatNVIDIA`, for chat models.

    Endpoint is required.

    Use this instead of passing `base_url` to a client constructor when the model's
    endpoint supports inference and not `/v1/models` listing.

    Use `base_url` when the model's endpoint supports `/v1/models` listing and inference
    on a known path, e.g. `/v1/chat/completions`.
    """
    if model.id in MODEL_TABLE:
        warnings.warn(
            f"Model {model.id} is already registered. "
            f"Overriding {MODEL_TABLE[model.id]}",
            UserWarning,
        )
    if not model.endpoint:
        raise ValueError(f"Model {model.id} does not have an endpoint.")
    MODEL_TABLE[model.id] = model


def lookup_model(name: str) -> Optional[Model]:
    """Lookup a model by name, using only the table of known models.

    The name is either:

    - Directly in the table
    - An alias in the table
    - Not found (`None`)

    Callers can check to see if the name was an alias by comparing the result's id field
    to the name they provided.
    """
    model = None
    if not (model := MODEL_TABLE.get(name)):
        for mdl in MODEL_TABLE.values():
            if mdl.aliases and name in mdl.aliases:
                model = mdl
                break
    return model


def determine_model(name: str) -> Optional[Model]:
    """Determine the model to use based on a name, using only the table of known models.

    Raise a warning if the model is found to be an alias of a known model.

    If the model is not found, return `None`.
    """
    if model := lookup_model(name):
        # all aliases are deprecated
        if model.id != name:
            warnings.warn(
                f"Model {name} is deprecated. Using {model.id} instead.", UserWarning
            )
    return model



from __future__ import annotations

import json
import warnings
from typing import Any, Dict, Generator, List, Literal, Optional, Sequence

from langchain_core.callbacks.manager import Callbacks
from langchain_core.documents import Document
from langchain_core.documents.compressor import BaseDocumentCompressor
from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    PrivateAttr,
)

from langchain_nvidia_ai_endpoints._common import _NVIDIAClient
from langchain_nvidia_ai_endpoints._statics import Model


class Ranking(BaseModel):
    index: int
    logit: float


_DEFAULT_MODEL_NAME: str = "nvidia/llama-3.2-nv-rerankqa-1b-v2"
_DEFAULT_BATCH_SIZE: int = 32


class NVIDIARerank(BaseDocumentCompressor):
    """
    LangChain Document Compressor that uses the NVIDIA NeMo Retriever Reranking API.
    """

    model_config = ConfigDict(
        validate_assignment=True,
    )

    _client: _NVIDIAClient = PrivateAttr()

    base_url: Optional[str] = Field(
        default=None,
        description="Base url for model listing an invocation",
    )

    top_n: int = Field(5, ge=0, description="The number of documents to return.")

    model: Optional[str] = Field(None, description="The model to use for reranking.")

    truncate: Optional[Literal["NONE", "END"]] = Field(
        default=None,
        description=(
            "Truncate input text if it exceeds the model's maximum token length. "
            "Default is model dependent and is likely to raise error if an "
            "input is too long."
        ),
    )

    max_batch_size: int = Field(
        _DEFAULT_BATCH_SIZE, ge=1, description="The maximum batch size."
    )

    default_headers: dict = Field(
        default_factory=dict,
        description="Default headers merged into all requests.",
    )

    extra_headers: dict = Field(
        default_factory=dict,
        description="Deprecated: Use 'default_headers' instead. "
        "Extra headers to include in the request.",
    )

    def __init__(
        self,
        *,
        nvidia_api_key: Optional[str] = None,
        api_key: Optional[str] = None,
        **kwargs: Any,
    ):
        """Create a new `NVIDIARerank` document compressor.

        This class provides access to a NVIDIA NIM for reranking. By default, it
        connects to a hosted NIM, but can be configured to connect to a local NIM
        using the `base_url` parameter.

        An API key is required to connect to the hosted NIM.

        Args:
            nvidia_api_key: The API key to use for connecting to the hosted NIM.
            api_key: Alternative to `nvidia_api_key`.
            **kwargs: Additional parameters passed to the underlying client.

        The recommended way to provide the API key is through the `NVIDIA_API_KEY`
        environment variable.

        Base URL:

        - Connect to a self-hosted model with NVIDIA NIM using the `base_url` arg to
            link to the local host at `localhost:8000`:

            ```python
            ranker = NVIDIARerank(base_url="http://localhost:8000/v1")
            ```

        Example:
            ```python
            from langchain_nvidia_ai_endpoints import NVIDIARerank
            from langchain_core.documents import Document

            query = "What is the GPU memory bandwidth of H100 SXM?"
            passages = [
                "The Hopper GPU is paired with the Grace CPU using NVIDIA's ultra-fast
                chip-to-chip interconnect, delivering 900GB/s of bandwidth, 7X faster
                than PCIe Gen5. This innovative design will deliver up to 30X higher
                aggregate system memory bandwidth to the GPU compared to today's fastest
                servers and up to 10X higher performance for applications running
                terabytes of data.",

                "A100 provides up to 20X higher performance over the prior generation
                and can be partitioned into seven GPU instances to dynamically adjust to
                shifting demands. The A100 80GB debuts the world's fastest memory
                bandwidth at over 2 terabytes per second (TB/s) to run the largest
                models and datasets.",

                "Accelerated servers with H100 deliver the compute power—along with 3
                terabytes per second (TB/s) of memory bandwidth per GPU and scalability
                with NVLink and NVSwitch™.",
            ]

            client = NVIDIARerank(
                model="nvidia/llama-3.2-nv-rerankqa-1b-v2",
                api_key="$API_KEY_REQUIRED_IF_EXECUTING_OUTSIDE_NGC"
            )

            response = client.compress_documents(
                query=query,
                documents=[Document(page_content=passage) for passage in passages]
            )

            print(f"Most relevant: {response[0].page_content}"
                f"Least relevant: {response[-1].page_content}"
            )

            # Most relevant: Accelerated servers with H100 deliver the compute
            # power—along with 3 terabytes per second (TB/s) of memory bandwidth per GPU
            # and scalability with NVLink and NVSwitch™.
            # Least relevant: A100 provides up to 20X higher performance over the prior
            # generation and can be partitioned into seven GPU instances to dynamically
            # adjust to shifting demands. The A100 80GB debuts the world's fastest
            # memory bandwidth at over 2 terabytes per second (TB/s) to run the largest
            # models and datasets.
            ```
        """

        super().__init__(**kwargs)

        # Handle backwards compatibility: if extra_headers is set but default_headers
        # is not, use extra_headers
        if self.extra_headers and not self.default_headers:
            warnings.warn(
                "The 'extra_headers' parameter is deprecated and will be removed "
                "in a future version. Please use 'default_headers' instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            self.default_headers = self.extra_headers

        # allow nvidia_base_url as an alternative for base_url
        base_url = kwargs.pop("nvidia_base_url", self.base_url)

        # allow nvidia_api_key as an alternative for api_key
        api_key = nvidia_api_key or api_key

        # Extract verify_ssl from kwargs, default to True
        verify_ssl = kwargs.pop("verify_ssl", True)

        self._client = _NVIDIAClient(
            **({"base_url": base_url} if base_url else {}),  # only pass if set
            mdl_name=self.model,
            default_hosted_model_name=_DEFAULT_MODEL_NAME,
            **({"api_key": api_key} if api_key else {}),  # only pass if set
            infer_path="{base_url}/ranking",
            cls=self.__class__.__name__,
            verify_ssl=verify_ssl,
        )

        # todo: only store the model in one place
        # the model may be updated to a newer name during initialization
        self.model = self._client.mdl_name

        # same for base_url
        self.base_url = self._client.base_url

    @property
    def available_models(self) -> List[Model]:
        """Get a list of available models that work with `NVIDIARerank`."""
        return self._client.get_available_models(self.__class__.__name__)

    @classmethod
    def get_available_models(
        cls,
        **kwargs: Any,
    ) -> List[Model]:
        """Get a list of available models that work with `NVIDIARerank`."""
        return cls(**kwargs).available_models

    def _prepare_payload(self, documents: List[str], query: str) -> Dict[str, Any]:
        """Prepare payload for both sync and async methods.

        Args:
            documents: List of document texts to rank
            query: Query text

        Returns:
            Payload dictionary
        """
        payload = {
            "model": self.model,
            "query": {"text": query},
            "passages": [{"text": passage} for passage in documents],
        }
        if self.truncate:
            payload["truncate"] = self.truncate
        return payload

    def _process_response(self, result: Dict[str, Any]) -> List[Ranking]:
        """Process response for both sync and async methods.

        Args:
            result: Parsed JSON response from the API

        Returns:
            List of rankings limited to top_n
        """
        rankings = result["rankings"]
        # todo: callback support
        return [Ranking(**ranking) for ranking in rankings[: self.top_n]]

    @staticmethod
    def _batch(ls: list, size: int) -> Generator[List[Document], None, None]:
        """Batch documents into chunks of specified size.

        Args:
            ls: List to batch
            size: Batch size

        Yields:
            Batches of documents
        """
        for i in range(0, len(ls), size):
            yield ls[i : i + size]

    def _process_batch_rankings(
        self,
        doc_batch: List[Document],
        rankings: List[Ranking],
        results: List[Document],
    ) -> None:
        """Process rankings for a batch of documents.

        Args:
            doc_batch: Batch of documents
            rankings: Rankings for the batch
            results: List to append processed documents to
        """
        for ranking in rankings:
            assert (
                0 <= ranking.index < len(doc_batch)
            ), "invalid response from server: index out of range"
            doc = doc_batch[ranking.index]
            doc.metadata["relevance_score"] = ranking.logit
            results.append(doc)

    @staticmethod
    def _sort_by_relevance(results: List[Document]) -> None:
        """Sort results by relevance score in descending order.

        Args:
            results: List of documents to sort in-place
        """
        results.sort(key=lambda x: x.metadata["relevance_score"], reverse=True)

    # todo: batching when len(documents) > endpoint's max batch size
    def _rank(self, documents: List[str], query: str) -> List[Ranking]:
        payload = self._prepare_payload(documents, query)
        response = self._client.get_req(
            payload=payload, extra_headers=self.default_headers
        )
        if response.status_code != 200:
            response.raise_for_status()
        # todo: handle errors
        return self._process_response(response.json())

    def compress_documents(
        self,
        documents: Sequence[Document],
        query: str,
        callbacks: Optional[Callbacks] = None,
    ) -> Sequence[Document]:
        """
        Compress documents using the NVIDIA NeMo Retriever Reranking microservice API.

        Args:
            documents: A sequence of documents to compress.
            query: The query to use for compressing the documents.
            callbacks: Callbacks to run during the compression process.

        Returns:
            A sequence of compressed documents.
        """
        if len(documents) == 0 or self.top_n < 1:
            return []

        doc_list = list(documents)
        results: List[Document] = []
        for doc_batch in self._batch(doc_list, self.max_batch_size):
            rankings = self._rank(
                query=query, documents=[d.page_content for d in doc_batch]
            )
            self._process_batch_rankings(doc_batch, rankings, results)

        # if we batched, we need to sort the results
        if len(doc_list) > self.max_batch_size:
            results.sort(key=lambda x: x.metadata["relevance_score"], reverse=True)

        return results[: self.top_n]

    async def _arank(self, documents: List[str], query: str) -> List[Ranking]:
        """Async version of _rank."""
        payload = self._prepare_payload(documents, query)
        response_text = await self._client.aget_req(
            payload=payload, extra_headers=self.default_headers
        )
        result = json.loads(response_text)
        return self._process_response(result)

    async def acompress_documents(
        self,
        documents: Sequence[Document],
        query: str,
        callbacks: Optional[Callbacks] = None,
    ) -> Sequence[Document]:
        """Async version of compress_documents."""
        if len(documents) == 0 or self.top_n < 1:
            return []

        doc_list = list(documents)
        results: List[Document] = []
        for doc_batch in self._batch(doc_list, self.max_batch_size):
            rankings = await self._arank(
                query=query, documents=[d.page_content for d in doc_batch]
            )
            self._process_batch_rankings(doc_batch, rankings, results)

        # if we batched, we need to sort the results
        if len(doc_list) > self.max_batch_size:
            results.sort(key=lambda x: x.metadata["relevance_score"], reverse=True)

        return results[: self.top_n]



"""
**LangChain NVIDIA AI Endpoints**

This comprehensive module integrates NVIDIA's state-of-the-art NIM endpoints,
featuring advanced models for conversational AI and semantic embeddings,
into the LangChain framework. It provides robust classes for seamless interaction
with AI models, particularly tailored for enriching conversational experiences
and enhancing semantic understanding in various applications.

**Features:**

1. **`ChatNVIDIA`:** This class serves as the primary interface for interacting
    with chat models. Users can effortlessly utilize advanced models like 'Nemotron'
    to engage in rich, context-aware conversations, applicable across diverse
    domains from customer support to interactive storytelling.

2. **`NVIDIAEmbeddings`:** The class offers capabilities to generate sophisticated
    embeddings using AI models. These embeddings are instrumental for tasks like
    semantic analysis, text similarity assessments, and contextual understanding,
    significantly enhancing the depth of NLP applications.

3. **`NVIDIARerank`:** This class provides an interface for reranking search results
    using AI models. Users can leverage this functionality to enhance search
    relevance and improve user experience in information retrieval systems.

4. **`NVIDIA`:** This class enables users to interact with large language models
    through a completions, or prompting, interface. Users can generate text
    completions, summaries, and other language model outputs using this class.
    This class is particularly useful for code generation tasks.

**Installation:**

Install this module easily using pip:

```bash
pip install langchain-nvidia-ai-endpoints
```

After setting up the environment, interact with NIM endpoints -

## Utilizing chat models:

```python
from langchain_nvidia import ChatNVIDIA

llm = ChatNVIDIA(model="nvidia/llama-3.1-nemotron-51b-instruct")
response = llm.invoke("Tell me about the LangChain integration.")
```

## Generating semantic embeddings:

Create embeddings useful in various NLP tasks:

```python
from langchain_nvidia import NVIDIAEmbeddings

embedder = NVIDIAEmbeddings(model="nvidia/nv-embedqa-e5-v5")
embedding = embedder.embed_query("Exploring AI capabilities.")
```

## Code completion using large language models:

```python
from langchain_nvidia import NVIDIA

llm = NVIDIA(model="meta/codellama-70b")
completion = llm.invoke("def hello_world():")
```
"""  # noqa: E501

from langchain_nvidia_ai_endpoints import *  # noqa: F403
from langchain_nvidia_ai_endpoints import __all__  # noqa: F401
